@@ -39,7 +39,6 @@ extern void	phase4(xfs_mount_t *);
 extern void	phase5(xfs_mount_t *);
 extern void	phase6(xfs_mount_t *);
 extern void	phase7(xfs_mount_t *);
-extern void	incore_init(xfs_mount_t *);
 
 #define		XR_MAX_SECT_SIZE	(64 * 1024)
 
@@ -339,19 +338,7 @@ process_args(int argc, char **argv)
 		usage();
 }
 
-void
-do_msg(int do_abort, char const *msg, va_list args)
-{
-	vfprintf(stderr, msg, args);
-
-	if (do_abort)  {
-		if (dumpcore)
-			abort();
-		exit(1);
-	}
-}
-
-void
+void __attribute__((noreturn))
 do_error(char const *msg, ...)
 {
 	va_list args;
@@ -359,20 +346,26 @@ do_error(char const *msg, ...)
 	fprintf(stderr, _("\nfatal error -- "));
 
 	va_start(args, msg);
-	do_msg(1, msg, args);
+	vfprintf(stderr, msg, args);
+	if (dumpcore)
+		abort();
+	exit(1);
 }
 
 /*
  * like do_error, only the error is internal, no system
  * error so no oserror processing
  */
-void
+void __attribute__((noreturn))
 do_abort(char const *msg, ...)
 {
 	va_list args;
 
 	va_start(args, msg);
-	do_msg(1, msg, args);
+	vfprintf(stderr, msg, args);
+	if (dumpcore)
+		abort();
+	exit(1);
 }
 
 void
@@ -383,7 +376,7 @@ do_warn(char const *msg, ...)
 	fs_is_dirty = 1;
 
 	va_start(args, msg);
-	do_msg(0, msg, args);
+	vfprintf(stderr, msg, args);
 	va_end(args);
 }
 
@@ -395,7 +388,7 @@ do_log(char const *msg, ...)
 	va_list args;
 
 	va_start(args, msg);
-	do_msg(0, msg, args);
+	vfprintf(stderr, msg, args);
 	va_end(args);
 }
 
@@ -542,11 +535,6 @@ main(int argc, char **argv)
 	bindtextdomain(PACKAGE, LOCALEDIR);
 	textdomain(PACKAGE);
 
-#ifdef XR_PF_TRACE
-	pf_trace_file = fopen("/tmp/xfs_repair_prefetch.trace", "w");
-	setvbuf(pf_trace_file, NULL, _IOLBF, 1024);
-#endif
-
 	temp_mp = &xfs_m;
 	setbuf(stdout, NULL);
 
@@ -699,9 +687,14 @@ main(int argc, char **argv)
 	calc_mkfs(mp);
 
 	/*
-	 * check sb filesystem stats and initialize in-core data structures
+	 * initialize block alloc map
 	 */
-	incore_init(mp);
+	init_bmaps(mp);
+	incore_ino_init(mp);
+	incore_ext_init(mp);
+
+	/* initialize random globals now that we know the fs geometry */
+	inodes_per_block = mp->m_sb.sb_inopblock;
 
 	if (parse_sb_version(&mp->m_sb))  {
 		do_warn(
@@ -729,6 +722,11 @@ main(int argc, char **argv)
 	}
 	timestamp(PHASE_END, 5, NULL);
 
+	/*
+	 * Done with the block usage maps, toss them...
+	 */
+	free_bmaps(mp);
+
 	if (!bad_ino_btree)  {
 		phase6(mp);
 		timestamp(PHASE_END, 6, NULL);
@@ -850,8 +848,7 @@ _("Note - stripe unit (%d) and width (%d) fields have been reset.\n"
 	if (verbose)
 		summary_report();
 	do_log(_("done\n"));
-#ifdef XR_PF_TRACE
-	fclose(pf_trace_file);
-#endif
+	pftrace_done();
+
 	return (0);
 }
